@@ -2,17 +2,17 @@
 use crate::internal_prelude::*;
 use crate::types as rt;
 
-pub enum Action {
-    Skip,
-    Term,
-}
+pub type ExtResult = rt::ExtResult<()>;
 
 pub trait Filter<JS: rt::JobStateValues, TS: rt::TaskStateValues> {
-    fn name(&self) -> &'static str;
+    fn name(&self) -> String {
+        String::from("no name")
+    }
     fn accept(
         &self,
         ctx: &rt::JobCtx<JS, TS>,
         task: &rt::Task,
         status: &rt::HttpStatus,
-    ) -> Action;
+        reader: Box<dyn io::Read + Sync + Send>
+    ) -> ExtResult;
 }
