@@ -7,15 +7,22 @@ pub use crate::{
     TaskExpander,
     types::{
         Link, LinkTarget, Job, JobCtx, Task, HttpStatus, JobStatus, JobUpdate,
-        select::predicate::Name,
-        select::document::Document,
-        flume::unbounded as ch_unbounded,
-        flume::bounded as ch_bounded,
-        flume::Receiver as ChReceiver,
-        flume::Sender as ChSender,
     },
     resolver::{
         AsyncHyperResolver,
     },
     config,
+    task_filters,
+    status_filters,
+    load_filters,
+    task_expanders,
+};
+
+pub use {
+    select::predicate::Name,
+    select::document::Document,
+    flume::unbounded as ch_unbounded,
+    flume::bounded as ch_bounded,
+    flume::Receiver as ChReceiver,
+    flume::Sender as ChSender,
 };
\ No newline at end of file
