@@ -1,22 +1,25 @@
 #[allow(unused_imports)]
 use crate::internal_prelude::*;
 use crate::types as rt;
+use crate::types::ExtError;
 
-#[derive(PartialEq)]
+#[derive(Clone, PartialEq, Debug, Copy )]
 pub enum Action {
     Accept,
     Skip,
-    Term,
 }
+pub type ExtResult = rt::ExtResult<Action>;
 
 pub trait Filter<JS: rt::JobStateValues, TS: rt::TaskStateValues> {
-    fn name(&self) -> &'static str;
+    fn name(&self) -> String {
+        String::from("no name")
+    }
     fn accept(
         &mut self,
         ctx: &mut rt::JobCtx<JS, TS>,
         task_seq_num: usize,
         task: &mut rt::Task,
-    ) -> Action;
+    ) -> ExtResult;
 }
 
 pub struct SelectiveTaskFilter<JS: rt::JobStateValues, TS: rt::TaskStateValues> {
@@ -33,13 +36,13 @@ impl<JS: rt::JobStateValues, TS: rt::TaskStateValues> SelectiveTaskFilter<JS, TS
 }
 
 impl<JS: rt::JobStateValues, TS: rt::TaskStateValues> Filter<JS, TS> for SelectiveTaskFilter<JS, TS> {
-    fn name(&self) -> &'static str {
+    fn name(&self) -> String {
         self.filter.name()
     }
-    fn accept(&mut self, ctx: &mut rt::JobCtx<JS, TS>, task_seq_num: usize, task: &mut rt::Task) -> Action {
+    fn accept(&mut self, ctx: &mut rt::JobCtx<JS, TS>, task_seq_num: usize, task: &mut rt::Task) -> ExtResult {
         let link_target = self.link_targets.iter().find(|target| target.to_string() == task.link.target.to_string());
         if link_target.is_none() {
-            return Action::Accept
+            return Ok(Action::Accept)
         }
         self.filter.accept( ctx, task_seq_num, task)
     }
@@ -74,23 +77,22 @@ pub struct HashSetDedup {
 }
 
 impl<JS: rt::JobStateValues, TS: rt::TaskStateValues> Filter<JS, TS> for SameDomain {
-    fn name(&self) -> &'static str {
-        "SameDomain"
-    }
-    fn accept(&mut self, ctx: &mut rt::JobCtx<JS, TS>, _: usize, task: &mut rt::Task) -> Action {
+    name!{}
+    fn accept(&mut self, ctx: &mut rt::JobCtx<JS, TS>, _: usize, task: &mut rt::Task) -> ExtResult {
         let domain = task.link.url.host_str().unwrap_or("_");
 
         let root_url =  &ctx.root_url;
         let root_domain = root_url.host_str().unwrap_or("");
         if root_domain.strip_prefix(&self.strip_prefix).unwrap_or(root_domain) == domain.strip_prefix(&self.strip_prefix).unwrap_or(domain)
         {
-            return Action::Accept;
+            return Ok(Action::Accept);
         }
-        Action::Skip
+        Ok(Action::Skip)
     }
 }
 
 impl SameDomain {
+    struct_name!{}
     pub fn new(www_allow: bool) -> Self {
         let mut strip_prefix = String::from("");
         if www_allow {
@@ -101,19 +103,18 @@ impl SameDomain {
 }
 
 impl<JS: rt::JobStateValues, TS: rt::TaskStateValues> Filter<JS, TS> for TotalPageBudget {
-    fn name(&self) -> &'static str {
-        "PageBudget"
-    }
-    fn accept(&mut self, _ctx: &mut rt::JobCtx<JS, TS>, _: usize, _: &mut rt::Task) -> Action {
+    name!{}
+    fn accept(&mut self, _ctx: &mut rt::JobCtx<JS, TS>, _: usize, _: &mut rt::Task) -> ExtResult {
         if self.budget >= self.allocated_budget {
-            return Action::Term;
+            return Err(ExtError::Term)
         }
         self.budget += 1;
-        Action::Accept
+        Ok(Action::Accept)
     }
 }
 
 impl TotalPageBudget {
+    struct_name!{}
     pub fn new(allocated_budget: usize) -> Self {
         Self {
             allocated_budget,
@@ -123,22 +124,21 @@ impl TotalPageBudget {
 }
 
 impl<JS: rt::JobStateValues, TS: rt::TaskStateValues> Filter<JS, TS> for LinkPerPageBudget {
-    fn name(&self) -> &'static str {
-        "LinkPerPageBudget"
-    }
-    fn accept(&mut self, _ctx: &mut rt::JobCtx<JS, TS>, seq_num: usize, _: &mut rt::Task) -> Action {
+    name!{}
+    fn accept(&mut self, _ctx: &mut rt::JobCtx<JS, TS>, seq_num: usize, _: &mut rt::Task) -> ExtResult {
         if seq_num > self.current_task_seq_num {
             self.links_within_current_task = 0;
         }
         self.links_within_current_task += 1;
         if self.links_within_current_task > self.allocated_budget {
-            return Action::Term;
+            return Err(ExtError::Term)
         }
-        Action::Accept
+        Ok(Action::Accept)
     }
 }
 
 impl LinkPerPageBudget {
+    struct_name!{}
     pub fn new(allocated_budget: usize) -> Self {
         Self {
             allocated_budget,
@@ -149,38 +149,36 @@ impl LinkPerPageBudget {
 }
 
 impl<JS: rt::JobStateValues, TS: rt::TaskStateValues> Filter<JS, TS> for PageLevel {
-    fn name(&self) -> &'static str {
-        "PageLevel"
-    }
-    fn accept(&mut self, _ctx: &mut rt::JobCtx<JS, TS>, _: usize, task: &mut rt::Task) -> Action {
+    name!{}
+    fn accept(&mut self, _ctx: &mut rt::JobCtx<JS, TS>, _: usize, task: &mut rt::Task) -> ExtResult {
         if task.level >= self.max_level {
-            return Action::Term;
+            return Err(ExtError::Term)
         }
-        Action::Accept
+        Ok(Action::Accept)
     }
 }
 
 impl PageLevel {
+    struct_name!{}
     pub fn new(max_level: usize) -> Self {
         Self { max_level }
     }
 }
 
 impl<JS: rt::JobStateValues, TS: rt::TaskStateValues> Filter<JS, TS> for HashSetDedup {
-    fn name(&self) -> &'static str {
-        "HashSetDedup"
-    }
-    fn accept(&mut self, _ctx: &mut rt::JobCtx<JS, TS>, _: usize, task: &mut rt::Task) -> Action {
+    name!{}
+    fn accept(&mut self, _ctx: &mut rt::JobCtx<JS, TS>, _: usize, task: &mut rt::Task) -> ExtResult {
         if self.visited.contains(task.link.url.as_str()) {
-            return Action::Skip;
+            return Ok(Action::Skip)
         }
 
         self.visited.insert(task.link.url.to_string());
-        Action::Accept
+        Ok(Action::Accept)
     }
 }
 
 impl HashSetDedup {
+    struct_name!{}
     pub fn new() -> Self {
         Self {
             visited: std::collections::HashSet::new(),
@@ -189,18 +187,17 @@ impl HashSetDedup {
 }
 
 impl<JS: rt::JobStateValues, TS: rt::TaskStateValues> Filter<JS, TS> for MaxRedirect {
-    fn name(&self) -> &'static str {
-        "MaxRedirect"
-    }
-    fn accept(&mut self, _ctx: &mut rt::JobCtx<JS, TS>, _: usize, task: &mut rt::Task) -> Action {
+    name!{}
+    fn accept(&mut self, _ctx: &mut rt::JobCtx<JS, TS>, _: usize, task: &mut rt::Task) -> ExtResult {
         if task.link.redirect > self.max_redirect {
-            return Action::Skip;
+            return Ok(Action::Skip)
         }
-        Action::Accept
+        Ok(Action::Accept)
     }
 }
 
 impl MaxRedirect {
+    struct_name!{}
     pub fn new(max_redirect: usize) -> Self {
         Self {
             max_redirect
