@@ -1,11 +1,12 @@
 #[allow(unused_imports)]
 use crate::internal_prelude::*;
-use crate::{types::*, status_filters, load_filters, hyper_utils, resolver::{Resolver, Adaptor as ResolverAdaptor}};
+use crate::{types::*, hyper_utils, resolver::{Resolver, Adaptor as ResolverAdaptor}};
 
-use bytes::{Buf, BufMut, BytesMut};
+use bytes::{Buf, BufMut, BytesMut, Bytes};
 use flate2::read::GzDecoder;
 use hyper::body::HttpBody;
 use rand::{Rng, thread_rng};
+use std::io::Read;
 
 pub(crate) type HttpClient<R> = hyper::Client<HttpConnector<R>>;
 pub(crate) type HttpConnector<R> = hyper_tls::HttpsConnector<hyper_utils::CountingConnector<hyper::client::HttpConnector<ResolverAdaptor<R>>>>;
@@ -19,6 +20,7 @@ pub(crate) struct TaskProcessor<JS: JobStateValues, TS: TaskStateValues, R: Reso
     status_filters: StatusFilters<JS, TS>,
     load_filters: LoadFilters<JS, TS>,
     task_expanders: Arc<TaskExpanders<JS, TS>>,
+    term_on_error: bool,
 
     tx: Sender<JobUpdate<JS, TS>>,
     tasks_rx: Receiver<Arc<Task>>,
@@ -35,12 +37,14 @@ impl<JS: JobStateValues, TS: TaskStateValues, R: Resolver> TaskProcessor<JS, TS,
         tasks_rx: Receiver<Arc<Task>>,
         parse_tx: Sender<ParserTask>,
         client_factory: Box<dyn ClientFactory<R> + Send + Sync + 'static>,
-        resolver: Arc<R>
+        resolver: Arc<R>,
+        term_on_error: bool
     ) -> TaskProcessor<JS, TS, R> {
         TaskProcessor {
             status_filters: job.rules.status_filters(),
             load_filters: job.rules.load_filters(),
             task_expanders: Arc::new(job.rules.task_expanders()),
+            term_on_error,
             job,
             tx,
             tasks_rx,
@@ -50,20 +54,26 @@ impl<JS: JobStateValues, TS: TaskStateValues, R: Resolver> TaskProcessor<JS, TS,
         }
     }
 
-    async fn read(&self, body: &mut hyper::Response<hyper::Body>) -> Result<BytesMut> {
+    async fn read(&self, body: &mut hyper::Response<hyper::Body>, encoding: String) -> anyhow::Result<Bytes> {
         let mut bytes = BytesMut::with_capacity(*self.job.settings.internal_read_buffer_size);
 
         while let Some(buf) = body.data().await {
-            let buf = buf.context("")?;
+            let buf = buf.context("error during reading")?;
             if buf.has_remaining() {
                 if bytes.len() + buf.len() > *self.job.settings.max_response_size as usize {
-                    return Err(anyhow!("max response size reached: {}", *self.job.settings.max_response_size).into());
+                    return Err(anyhow!("max response size reached: {}", *self.job.settings.max_response_size));
                 }
                 bytes.put(buf);
             }
         }
 
-        Ok(bytes)
+        if encoding.contains("gzip") || encoding.contains("deflate") {
+            let mut buf: Vec<u8> = vec![];
+            let _ = GzDecoder::new(bytes.reader()).read_to_end(&mut buf)?;
+            return Ok(Bytes::from(buf))
+        }
+
+        Ok(bytes.freeze())
     }
 
     async fn resolve(
@@ -112,7 +122,7 @@ impl<JS: JobStateValues, TS: TaskStateValues, R: Resolver> TaskProcessor<JS, TS,
 
         let resp;
         tokio::select! {
-            r = client.request(req.body(hyper::Body::default()).unwrap()) => {
+            r = client.request(req.body(hyper::Body::default()).context("could not construct http request")?) => {
                 resp = r;
             },
             _ = timeout => {
@@ -131,22 +141,22 @@ impl<JS: JobStateValues, TS: TaskStateValues, R: Resolver> TaskProcessor<JS, TS,
             status_metrics,
         };
 
-        let mut term_by = None;
         for filter in self.status_filters.iter() {
             let r = filter.accept(&mut self.job.ctx, &task, &status);
             match r {
-                status_filters::Action::Term => {
-                    term_by = Some(filter.name());
-                    break
+                Err(ExtError::Term) => {
+                    return Err(Error::StatusFilterTerm {name: filter.name()})
                 },
-                status_filters::Action::Skip => {}
+                Err(ExtError::Other(err)) => {
+                    trace!("error in status filter {}: {:#}", filter.name(), &err);
+                    if self.term_on_error {
+                        return Err(Error::StatusFilterTermByError {name: filter.name(), source: err})
+                    }
+                },
+                Ok(_) => {},
             }
         }
 
-        if term_by.is_some() {
-            return Err(Error::FilterTerm {name: term_by.unwrap().into()})
-        }
-
         Ok((status, resp))
     }
 
@@ -156,81 +166,96 @@ impl<JS: JobStateValues, TS: TaskStateValues, R: Resolver> TaskProcessor<JS, TS,
         status: &HttpStatus,
         stats: &hyper_utils::Stats,
         mut resp: hyper::Response<hyper::Body>,
-    ) -> Result<(LoadData, Box<dyn std::io::Read + Sync + Send>)> {
+    ) -> Result<(LoadData, Box<dyn io::Read + Sync + Send>)> {
         let mut load_metrics = LoadMetrics {..Default::default()};
 
-        let body = self.read(&mut resp).await.with_context(|| "cannot read http get response")?;
+        let enc = String::from(resp.headers()
+            .get(http::header::CONTENT_ENCODING)
+            .map_or("",|h| h.to_str().unwrap_or("")));
+
+        let body = self.read(&mut resp, enc).await.with_context(|| "cannot read http get response")?;
 
         load_metrics.read_size = stats.read();
         load_metrics.write_size = stats.write();
 
-        let mut enc = String::from("");
-        let content_encoding_header = resp.headers().get(http::header::CONTENT_ENCODING);
-        if content_encoding_header.is_some() {
-            let s = content_encoding_header.unwrap().to_str();
-            if s.is_ok() {
-                enc = String::from(s.unwrap());
-            }
-        }
-
-        let reader;
-        if enc.contains("gzip") || enc.contains("deflate") {
-            reader = Box::new(GzDecoder::new(body.reader())) as Box<dyn std::io::Read + Sync + Send>
-        } else {
-            reader = Box::new(body.reader()) as Box<dyn std::io::Read + Sync + Send>
-        }
-
         load_metrics.load_dur = status.started_processing_on.elapsed();
 
-        let mut term_by = None;
         for filter in self.load_filters.iter() {
-            let r = filter.accept(&self.job.ctx, &task, &status);
+            let r = filter.accept(&self.job.ctx, &task, &status, Box::new(body.clone().reader()));
             match r {
-                load_filters::Action::Term => {
-                    term_by = Some(filter.name());
-                    break
-                }
-                load_filters::Action::Skip => {}
+                Err(ExtError::Term) => {
+                    return Err(Error::LoadFilterTerm {name: filter.name()})
+                },
+                Err(ExtError::Other(err)) => {
+                    trace!("error in status filter {}: {:#}", filter.name(), &err);
+                    if self.term_on_error {
+                        return Err(Error::LoadFilterTermByError {name: filter.name(), source: err})
+                    }
+                },
+                Ok(_) => {},
             }
         }
 
-        if term_by.is_some() {
-            return Err(Error::FilterTerm {name: term_by.unwrap().into()})
-        }
-
         let load_data = LoadData{
             metrics: load_metrics,
         };
-        Ok((load_data, reader))
+        Ok((load_data, Box::new(body.reader())))
     }
 
     async fn follow(
         &mut self,
         task: Arc<Task>,
         status: HttpStatus,
-        reader: Box<dyn std::io::Read + Sync + Send>,
+        reader: Box<dyn io::Read + Sync + Send>,
     ) -> Result<FollowData> {
         let (parse_res_tx, parse_res_rx) = bounded_ch::<ParserResponse>(1);
 
+        let task_state = Arc::new(Mutex::new(Some(TS::default())));
+        let links = Arc::new(Mutex::new(Some(Vec::<Link>::new())));
+
         let payload = {
             let task = Arc::clone(&task);
             let task_expanders = Arc::clone(&self.task_expanders);
+            let term_on_error = self.term_on_error;
             let mut job_ctx = self.job.ctx.clone();
+            let task_state = Arc::clone(&task_state);
+            let links = Arc::clone(&links);
 
-            move || -> Result<(FollowData, Vec<Arc<Link>>)> {
+            move || -> Result<FollowData> {
                 let t = Instant::now();
 
                 let document = select::document::Document::from_read(reader).context("cannot read html document")?;
 
-                for h in task_expanders.iter() {
-                    h.expand(&mut job_ctx, &task, &status, &document);
+                for expander in task_expanders.iter() {
+                    match expander.expand(&mut job_ctx, &task, &status, &document) {
+                        Ok(_) => {},
+                        Err(ExtError::Term) => {
+                            return Err(Error::TaskExpanderTerm {name: expander.name()})
+                        },
+                        Err(ExtError::Other(err)) => {
+                            if term_on_error {
+                                return Err(Error::TaskExpanderTermByError {name: expander.name(), source: err})
+                            }
+                            trace!("error in task expander {}: {:#}", expander.name(), &err);
+                        }
+                    }
                 }
 
-                Ok((FollowData{
+                {
+                    let mut links = links.lock().unwrap();
+                    *links = Some(job_ctx._consume_links());
+                }
+
+                {
+                    let mut task_state = task_state.lock().unwrap();
+                    *task_state = Some(job_ctx.task_state);
+                }
+
+                Ok(FollowData{
                     metrics: FollowMetrics{
                         parse_dur: t.elapsed(),
                     },
-                }, job_ctx.consume_links()))
+                })
             }
         };
 
@@ -241,8 +266,15 @@ impl<JS: JobStateValues, TS: TaskStateValues, R: Resolver> TaskProcessor<JS, TS,
         }).await;
 
         let parser_response = parse_res_rx.recv_async().await.context("cannot follow html document")?;
-        let (follow_data, links) = parser_response.res?;
-        self.job.ctx.push_arced_links(links);
+        let follow_data = parser_response.payload?;
+        {
+            let mut task_state = task_state.lock().unwrap();
+            self.job.ctx.task_state = task_state.take().unwrap();
+        }
+        {
+            let mut links = links.lock().unwrap();
+            self.job.ctx.push_links(links.take().unwrap());
+        }
         Ok(follow_data)
     }
 
@@ -263,47 +295,54 @@ impl<JS: JobStateValues, TS: TaskStateValues, R: Resolver> TaskProcessor<JS, TS,
         }
 
         if task.link.target == LinkTarget::Head || task.link.target == LinkTarget::HeadLoad || task.link.target == LinkTarget::HeadFollow {
-            let status_r = self.status(Arc::clone(&task), &client, true).await;
-            if status_r.is_err() {
-                status.head_status = StatusResult::Err(status_r.err().unwrap());
-                return Ok(status);
+            match self.status(Arc::clone(&task), &client, true).await {
+                Ok((head_status, _)) => {
+                    status.head_status = StatusResult::Ok(head_status);
+                }
+                Err(err) => {
+                    status.head_status = StatusResult::Err(err);
+                    return Ok(status);
+                }
             }
-            let (head_status, _) = status_r.unwrap();
-            status.head_status = StatusResult::Ok(head_status);
         };
         if task.link.target == LinkTarget::Head {
             return Ok(status);
         }
 
-        let status_r = self.status(Arc::clone(&task), &client, false).await;
-        if status_r.is_err() {
-            status.status = StatusResult::Err(status_r.err().unwrap());
-            return Ok(status);
-        }
-
-        let (get_status, resp) = status_r.unwrap();
+        let (get_status, resp) = match self.status(Arc::clone(&task), &client, false).await{
+            Err(err) => {
+                status.status = StatusResult::Err(err);
+                return Ok(status);
+            },
+            Ok((get_status, resp)) => (get_status, resp)
+        };
 
         let load_r = self.load(Arc::clone(&task), &get_status, stats, resp).await;
         status.status = StatusResult::Ok(get_status.clone());
 
-        if load_r.is_err() {
-            status.load_data = LoadResult::Err(load_r.err().unwrap());
-            return Ok(status);
-        }
+        let reader= match load_r {
+            Err(err) => {
+                status.load_data = LoadResult::Err(err);
+                return Ok(status);
+            }
+            Ok((load_data, reader)) => {
+                status.load_data = LoadResult::Ok(load_data);
+                reader
+            }
+        };
 
-        let (load_data, reader) = load_r.unwrap();
-        status.load_data = LoadResult::Ok(load_data);
         if task.link.target == LinkTarget::Load || task.link.target == LinkTarget::HeadLoad {
             return Ok(status);
         }
 
-        let follow_r = self.follow(Arc::clone(&task), get_status, reader).await;
-        if follow_r.is_err() {
-            status.follow_data = FollowResult::Err(follow_r.err().unwrap());
-            return Ok(status);
+        match self.follow(Arc::clone(&task), get_status, reader).await {
+            Ok(follow_data) => {
+                status.follow_data = FollowResult::Ok(follow_data);
+            },
+            Err(err) => {
+                status.follow_data = FollowResult::Err(err);
+            }
         }
-        let follow_data = follow_r.unwrap();
-        status.follow_data = FollowResult::Ok(follow_data);
         Ok(status)
     }
 
