@@ -1,7 +1,6 @@
 #[allow(unused_imports)]
 use crate::internal_prelude::*;
-pub use select;
-pub use flume;
+
 use crate::{
     task_filters,
     status_filters,
@@ -75,14 +74,49 @@ pub enum Error {
     Other(#[from] anyhow::Error),
     #[error("timeout during loading")]
     LoadTimeout,
-    #[error("terminated by filter {name:?}")]
-    FilterTerm{
+    #[error("terminated by status filter {name:?}")]
+    StatusFilterTerm {
         name: String
-    }
+    },
+    #[error("terminated by error in status filter(term_by_error=true) {name:?}")]
+    StatusFilterTermByError {
+        name: String,
+        #[source]
+        source: anyhow::Error
+    },
+    #[error("terminated by load filter {name:?}")]
+    LoadFilterTerm {
+        name: String
+    },
+    #[error("terminated by error in load filter(term_by_error=true) {name:?}")]
+    LoadFilterTermByError {
+        name: String,
+        #[source]
+        source: anyhow::Error
+    },
+    #[error("terminated by task expander {name:?}")]
+    TaskExpanderTerm {
+        name: String
+    },
+    #[error("terminated by error in task expander(term_by_error=true) {name:?}")]
+    TaskExpanderTermByError {
+        name: String,
+        #[source]
+        source: anyhow::Error
+    },
 }
 
 pub type Result<T> = anyhow::Result<T, Error>;
 
+#[derive(Error, Debug)]
+pub enum ExtError {
+    #[error(transparent)]
+    Other(#[from] anyhow::Error),
+    #[error("terminated")]
+    Term,
+}
+pub type ExtResult<T> = std::result::Result<T, ExtError>;
+
 #[derive(Error, Debug)]
 pub enum JobError {
     #[error("job finished by soft timeout")]
@@ -91,7 +125,7 @@ pub enum JobError {
     JobFinishedByHardTimeout,
 }
 
-#[derive(Clone, PartialEq, Debug )]
+#[derive(Clone, PartialEq, Debug, Copy )]
 pub enum LinkTarget {
     JustResolveDNS = 0,
     Head = 1,
@@ -219,13 +253,13 @@ pub struct JobUpdate<JS: JobStateValues, TS: TaskStateValues> {
 }
 
 pub struct ParserTask {
-    pub payload: Box<dyn FnOnce() -> Result<(FollowData, Vec<Arc<Link>>)> + Send + 'static>,
+    pub payload: Box<dyn FnOnce() -> Result<FollowData> + Send + 'static>,
     pub time: Instant,
     pub res_tx: Sender<ParserResponse>
 }
 
 pub struct ParserResponse {
-    pub res: Result<(FollowData, Vec<Arc<Link>>)>,
+    pub payload: Result<FollowData>,
     pub wait_time: Duration,
     pub work_time: Duration
 }
@@ -245,8 +279,8 @@ pub struct JobCtx<JS, TS> {
     pub started_at : Instant,
     pub root_url : Url,
     pub job_state: Arc<Mutex<JS>>,
-    pub task_state: Arc<Mutex<TS>>,
-    links: Vec<Arc<Link>>,
+    pub task_state: TS,
+    links: Vec<Link>,
 }
 
 impl<JS: JobStateValues, TS: TaskStateValues> JobCtx<JS, TS> {
@@ -255,7 +289,7 @@ impl<JS: JobStateValues, TS: TaskStateValues> JobCtx<JS, TS> {
             started_at: Instant::now(),
             root_url,
             job_state: Arc::new(Mutex::new(job_state)),
-            task_state: Arc::new(Mutex::new(task_state)),
+            task_state,
             links: vec![],
         }
     }
@@ -272,14 +306,14 @@ impl<JS: JobStateValues, TS: TaskStateValues> JobCtx<JS, TS> {
     }
 
     pub fn push_links(&mut self, links: Vec<Link>) {
-        self.links.extend(links.into_iter().map(|link|Arc::new(link)))
-    }
-
-    pub(crate) fn push_arced_links(&mut self, links: Vec<Arc<Link>>) {
-        self.links.extend(links)
+        self.links.extend(links.into_iter())
     }
 
     pub fn consume_links(&mut self) -> Vec<Arc<Link>> {
+        self.links.drain(0..).map(Arc::new).collect()
+    }
+
+    pub(crate) fn _consume_links(&mut self) -> Vec<Link> {
         self.links.drain(0..).collect()
     }
 }
